"lprq" <-
function(x, y, h, tau = .5, m = 50)
{
	require(quantreg)
        xx <- seq(min(x),max(x),length=m)
        fv <- xx
        der <- xx
        for(i in 1:length(xx)) {
                z <- x - xx[i]
                wx <- dnorm(z/h)
                r <- rq(y~z, weights=wx, tau=tau, ci=FALSE)
                fv[i] <- r$coef[1.]
                der[i] <- r$coef[2.]
        }
        list(xx = xx, fv = fv, der = der)
}

"[.terms" <- function (termobj, i) 
{
    resp <- if (attr(termobj, "response")) 
        termobj[[2]]
    else NULL
    newformula <- attr(termobj, "term.labels")[i]
    if (length(newformula) == 0) 
        newformula <- 1
    newformula <- reformulate(newformula, resp)
    environment(newformula) <- environment(termobj)
    terms(newformula, specials = names(attr(termobj, "specials")))
}
"rqss" <-
function (formula, tau = 0.5, data = parent.frame(), weights, 
    na.action, method = "fn", contrasts = NULL, ...) 
{
    call <- match.call()
    m <- match.call(expand = FALSE)
    temp <- c("", "formula", "data", "weights", "na.action")
    m <- m[match(temp, names(m), nomatch = 0)]
    m[[1]] <- as.name("model.frame")
    special <- "qss"
    Terms <- if (missing(data)) 
        terms(formula, special)
    else terms(formula, special, data = data)
    qssterms <- attr(Terms, "specials")$qss
    dropx <- NULL
    if (length(qssterms)) {
        tmpc <- untangle.specials(Terms, "qss")
        ord <- attr(Terms, "order")[tmpc$terms]
        if (any(ord > 1)) 
            stop("qss can not be used in an interaction")
        dropx <- tmpc$terms
        if (length(dropx)) 
            Terms <- Terms[-dropx]
        attr(Terms, "specials") <- tmpc$vars
    }
    m$formula <- Terms
    m <- eval(m, parent.frame())
    weights <- model.extract(m, weights)
    process <- (tau < 0 || tau > 1)
    Y <- model.extract(m, response)
    X <- model.matrix(Terms, m, contrasts)
    p <- ncol(X)
    if (length(qssterms) > 0) {
        F <- as.matrix.csr(X)
        qss <- lapply(tmpc$vars, function(u) eval(parse(text = u), 
            data))
        mqss <- length(qss)
        ncA <- rep(0, mqss + 1)
        nrA <- rep(0, mqss + 1)
        nrR <- rep(0, mqss + 1)
        for (i in 1:mqss) {
            F <- cbind(F, qss[[i]]$F)
            ncA[i + 1] <- ncol(qss[[i]]$A)
            nrA[i + 1] <- nrow(qss[[i]]$A)
            nrR[i + 1] <- ifelse(is.null(nrow(qss[[i]]$R)), 0, 
                nrow(qss[[i]]$R))
        }
        A <- as.matrix.csr(0, sum(nrA), sum(ncA))
        if (sum(nrR) > 0) {
            R <- as.matrix.csr(0, sum(nrR), sum(ncA))
            nrR <- cumsum(nrR)
        }
        ncA <- cumsum(ncA)
        nrA <- cumsum(nrA)
        for (i in 1:mqss) {
            A[(1 + nrA[i]):nrA[i + 1], (1 + ncA[i]):ncA[i + 1]] <- qss[[i]]$A
            if (nrR[i] < nrR[i + 1]) 
                R[(1 + nrR[i]):nrR[i + 1], (1 + ncA[i]):ncA[i + 
                  1]] <- qss[[i]]$R
        }
        A <- cbind(as.matrix.csr(0, nrA[mqss + 1], p), 
            A)
        if (nrR[mqss + 1] > 0) {
            R <- cbind(as.matrix.csr(0, nrR[mqss + 1], p), 
                R)
            r <- rep(0, nrR[mqss + 1])
        }
        else {
            R <- NULL
            r <- NULL
        }
        X <- rbind(F, A)
        Y <- c(Y, rep(0, nrow(A)))
	rhs <- t(rbind((1 - tau) * F, 0.5 * A))%*%rep(1,nrow(X))
	XpX <- t(X)%*%X
	nnzdmax <- XpX@ia[length(XpX@ia)]-1
	nsubmax <- max(nnzdmax,floor(1e3 + exp(-1.6) * nnzdmax^1.2))
	nnzlmax <- floor(2e5 - 2.8*nnzdmax + 7e-4 * nnzdmax^2)
	tmpmax <- floor(1e5+exp(-12.1)*nnzdmax^2.35)
        fit <- if (length(r) > 0) 
	      rqss.fit(X, Y, tau = tau, rhs = rhs, method = "sfnc", R = R,
                r = r, nsubmax = nsubmax, nnzlmax = nnzlmax, tmpmax = tmpmax)
        else 
	      rqss.fit(X, Y, tau = tau, rhs = rhs, method = "sfn",
               nsubmax = nsubmax, nnzlmax = nnzlmax, tmpmax = tmpmax)
        for(i in 1:mqss){
		ML <- p+1+ncA[i]
                MU <- p+ncA[i+1]
                qss[[i]] <- cbind(qss[[i]]$x$x,qss[[i]]$x$y,
                        c(0,fit$coef[ML:MU]))
		}
	fit$qss <- qss
    }
    else {
        fit <- if (length(weights)) 
            rq.wfit(X, Y, tau = tau, weights, method, ...)
        else rq.fit(X, Y, tau = tau, method, ...)
    }
    fit$terms <- Terms
    fit$call <- call
    fit$tau <- tau
    attr(fit, "na.message") <- attr(m, "na.message")
    class(fit) <-  "rqss"
    fit
}
"rqss.fit" <-
function (x, y, tau = 0.5, method = "sfn", ...) 
{
    fit <- switch(method, 
	sfn = rq.fit.sfn(x, y, tau = tau, ...), 
        sfnc = rq.fit.sfnc(x, y, tau = tau, ...), {
            what <- paste("rq.fit.", method, sep = "")
            if (exists(what, mode = "function")) 
                (get(what, mode = "function"))(x, y, ...)
            else stop(paste("unimplemented method:", method))
        })
    fit$contrasts <- attr(x, "contrasts")
    fit
}
"untangle.specials" <-
function (tt, special, order = 1) 
{
    spc <- attr(tt, "specials")[[special]]
    if (length(spc) == 0) 
        return(list(vars = character(0), terms = numeric(0)))
    facs <- attr(tt, "factor")
    fname <- dimnames(facs)
    ff <- apply(facs[spc, , drop = FALSE], 2, sum)
    list(vars = (fname[[1]])[spc], terms = seq(ff)[ff & match(attr(tt, 
        "order"), order, nomatch = 0)])
}
"qss" <-
function(x, constraint = "N", lambda = 1, w=rep(1,length(x))){
   if(is.list(x)) 
        if(all(!is.na(match(c("x", "y"), names(x))))) 
		qss <- qss2(x$x,x$y,constraint=constraint, lambda=lambda, w=w)
        else stop("called with list lacking xy elements")
   else if(is.matrix(x))
	if(ncol(x)==2)
		qss <- qss2(x[,1],x[,2],constraint=constraint, lambda=lambda, w=w)
        else stop("matrix argument to qss must have two columns")
   else if(is.vector(x))
	qss <- qss1(x,constraint=constraint, lambda=lambda, w=w)
   else
        stop("x argument to qss invalid")
   qss
}
"qss2" <-
function(x, y, constraint = "N", lambda = 1, w=rep(1,length(x))){
#
# Sparse Additive Quantile Smoothing Spline Models - Bivariate (Triogram) Module
#
# This function returns a structure intended to make model.matrix for a bivariate
# nonparametric component of a model formula specified by a call to rqss().  A sparse form
# of the Frisch Newton algorithm is eventually called to compute the estimator.
# An optional convexity/concavity constraint can be specified.  If
# the formula consists of a single qss component then the estimator solves the
# following variational problem:
#
#       min sum rho_tau (y_i - g(x_i)) + lambda V(grad(g))
#
# where V(f) denotes the total variation of the function f.  The solution is a piecewise
# linear function on the Delaunay triangulation formed by the observed (x_i,y_i) points.  
# Additive models can consist
# of several components of this form plus partial linear and univariate qss components.
# To resolve the identifiability problem we delete the first column of the qss design
# components.  On return F contains the fidelity portion of the design, A the penalty
# contribution of the design, R the constraint portion, and r the rhs of the constraints.
#
# Constraints are specified by the constraint argument: 
#
#	N	none
#	U	convex
#	C	concave
#
# Author:  Roger Koenker   April 2, 2003
#
# For a prototype see triogram in ~roger/projects/tv/cobar/.RData on ysidro.
#
# Warning:   Under development...todo:
#
#       o  weights
#       o  dummy x's
#       o  tau's
#       o  lambda's
#       o  ...
#
   require(SparseM)
   require(tripack)
#
   n <- length(x)
   if(n != length(y))
          stop("xy lengths do not match")
   f <- triogram.fidelity(x,y)
   F <- f$F
   A <- triogram.penalty(f$x, f$y)

switch(constraint,
        V = {   R <- A;
                r <- rep(0,nrow(R))
                },
        C = {   R <- -A;
                r <- rep(0,nrow(R))
                },
        N = { R=NULL; r=NULL}
        )
   list(x = list(x=f$x, y=f$y), F=F[,-1], A=lambda*A[,-1], R=R[,-1], r=r)
}

"qss1" <-
function(x, constraint = "N", lambda = 1, w=rep(1,length(x))){
#
# Sparse Additive Quantile Smoothing Spline Models - Univariate Module
#
# This function returns a structure intended to make model.matrix for a univariate
# nonparametric component of a model formula specified by a call to rq().  A sparse form
# of the Frisch Newton algorithm is eventually called to compute the estimator.
# Optional monotonicity and/or convexity/concavity constraints can be specified.  If
# the formula consists of a single qss component then the estimator solves the
# following variational problem:
#
#       min sum rho_tau (y_i - g(x_i)) + lambda V(g')
#
# where V(f) denotes the total variation of the function f.  The solution is a piecewise
# linear function with "knots" at the observed x_i points.  Additive models can consist
# of several components of this form plus partial linear and triogram components.
# To resolve the identifiability problem we delete the first column of the qss design
# components.  On return F contains the fidelity portion of the design, A the penalty
# contribution of the design, R the constraint portion, and r the rhs of the constraints.
#
# Constraints are specified by the constraint argument: 
#
#	N	none
#	I	monotone increasing
#	D	monotone decreasing
#	V	convex
#	C	concave
#	CI	concave and monotone increasing
#	...	etc
#
# Author:  Roger Koenker   February 27, 2003
#
# Warning:   Under development...todo:
#
#       o  weights
#       o  dummy x's
#       o  tau's
#       o  lambda's
#       o  ...
#
# See sqss for a prototype version of this function.
#
   require(SparseM)
#
   xun <- unique(x[order(x)])
   h <- diff(xun)
   nh <- length(h)
   nx <- length(x)
   p <- nh + 1
   B <- new("matrix.csr", ra = c(rbind(-1/h,1/h)), ja = as.integer(c(rbind(1:nh,2:(nh+1)))), 
		ia = as.integer(2*(1:(nh+1))-1), dimension = as.integer(c(nh,nh+1)))
   makeD <- function(p){ new("matrix.csr", ra = c(rbind(rep(-1,(p-1)),rep(1,(p-1)))), 
		ja = as.integer(c(rbind(1:(p-1),2:p))), ia = as.integer(2*(1:p)-1), 
		dimension = as.integer(c(p-1,p)))
		}
   D <- makeD(nh)
   A <- D %*% B
   if(length(xun) == length(x)){
       F <- new("matrix.csr", ra = rep(1,nx), ja = as.integer(rank(x)),
                ia = 1:(nx+1), dimension = as.integer(c( nx, nx)))
        }
   else {
       F <- new("matrix.csr", ra = rep(1,nx), ja = as.integer(factor(x)),
                ia = 1:(nx+1), dimension = as.integer(c( nx, length(xun))))
       }
   switch(constraint,
        V = {   R <- A;
                r <- rep(0,nrow(R))
                },
        C = {   R <- -A;
                r <- rep(0,nrow(R))
                },
        I = {   R <- makeD(p)
                r <- rep(0,p-1)
                },
        D = {   R <- -makeD(p)
                r <- rep(0,p-1)
                },
        VI = {  R <- makeD(p)
		R <- rbind(R,A)
                r <- rep(0,nrow(R))
		},
        VD = {  R <- -makeD(p)
		R <- rbind(R,A)
                r <- rep(0,nrow(R))
		},
        CI = {  R <- makeD(p)
		R <- rbind(R,-A)
                r <- rep(0,nrow(R))
		},
        CD = {  R <- -makeD(p)
		R <- rbind(R,-A)
                r <- rep(0,nrow(R))
		},
	N = { R=NULL; r=NULL}
	)
   list(x = list(x=xun), F=F[,-1], A=lambda*A[,-1], R=R[,-1], r=r)
}
"plot.qss1" <-
function(x, ...)
{
#plot(x[,1],x[,2],type="n",xlab="x",ylab="fit")
lines(x[,1],x[,2], ...)
}
"plot.qss2" <-
function (x, ...)
{
require(akima)
require(tripack)

        y <- x[,2]
        z <- x[,3]
        x <- x[,1]
	par(mfrow=c(1,2))
        plot(x,y,type="n",axes=TRUE,xlab="x",ylab="y")
        contour(interp(x,y,z),add=TRUE,frame.plot=TRUE, axes=TRUE, ...)
        tri <- tri.mesh(x,y)
        convex.hull(tri,plot.it=TRUE,add=TRUE)
	persp(interp(x,y,z,),theta=-40,phi=20,xlab="x",ylab="y",zlab="z", ...)
}
"plot.rqss" <-
function(x, ...) 
{
m <- length(x$qss)
for( i in 1:m){
        if(ncol(x$qss[[i]])==3){
		x$qss[[i]][,3] <- x$coef[1] + x$qss[[i]][,3]
            	plot.qss2(x$qss[[i]], ...)
	   }
        else if(ncol(x$qss[[i]])==2){
		x$qss[[i]][,2] <- x$coef[1] + x$qss[[i]][,2]
            	plot.qss1(x$qss[[i]], ...)
	   }
        else
                stop("invalid fitted qss object")
        }
}

"triogram.fidelity" <- function (x,y,ndum=0) 
{
#Make fidelity block of the triogram design in sparse matrix.csr form
#The rather esoteric match call identifies and handles duplicated xy points
n <- length(x)
A <- as.data.frame(cbind(x,y))
dupA <- duplicated(A)
if(any(dupA)){
  x <- x[!dupA]
  y <- y[!dupA]
  J <- match(do.call("paste",c(A,"\r")),do.call("paste",c(A[!dupA,],"\r")))
  z <- new("matrix.csr",ra=rep(1,n), ja=J, ia=1:(n+1),dim=as.integer(c(n,max(J))))
  }
else
  z <- as.matrix.csr(diag(n))
#Augment with dummy vertices, if any...
if(ndum > 0){
        u <- runif(ndum); v <- runif(ndum)
        xd <- min(x) + u * (max(x)-min(x))
        yd <- min(y) + v * (max(y)-min(y))
        T <- tri.mesh(x,y)
        s <- in.convex.hull(T,xd,yd)
        x <- c(x,xd[s])
        y <- c(y,yd[s])
        ndum <- sum(s)
        zdum <- matrix.csr(0,n,ndum)
        z <- cbind(z,zdum)
        }
list(x=x,y=y,F=z)
}
"triogram.penalty" <- function (x, y, eps = .Machine$double.eps) 
{
    n <- length(x)
    tri <- tri.mesh(x, y)
    bnd <- on.convex.hull(tri,x,y)
    q <- length(tri$tlist)
    m <- 13 * n
    z <- .Fortran("penalty", as.integer(n), as.integer(m), as.integer(q), 
        as.double(x), as.double(y), as.integer(bnd),as.integer(tri$tlist), 
        as.integer(tri$tlptr), as.integer(tri$tlend), rax = double(m), 
	jax = integer(m), ned = integer(1), as.double(eps), ierr = integer(1),
	PACKAGE = "nprq")[c("rax", "jax", "iax", "ned", "ierr")]
    if (z$ierr == 1) 
        stop("collinearity in ggap")
    nnz <- 4 * z$ned
    ra <- z$rax[1:nnz]
    ja <- z$jax[1:nnz] 
    ia <- as.integer(1 + 4 * (0:z$ned))
    dim <- as.integer(c(z$ned, n))
    new("matrix.csr",ra=ra,ja=ja,ia=ia,dimension=dim)
}


#-------------------------------------------------------------------------------
".First.lib" <-
function(lib, pkg) {
   require(SparseM)
   library.dynam("nprq", pkg, lib)
   print("nprq loaded")
}
#-------------------------------------------------------------------------------
"rq.fit.sfn" <- function(a,y,tau=.5, rhs = (1-tau)*c(t(a) %*% rep(1,length(y))),
	nsubmax,tmpmax,nnzlmax,cachsz=64,small=1.e-6,maxiter=100,warn.mesg=TRUE,...){
#################################################################
# Interface for a sparse implementation of LMS interior point method
#	a -- structure of the design matrix A' stored in csr format
#	y -- pseudo response vector
#	m -- column dimension of A' in full mode
#       rhs -- the rhs of the dual problem -- specify at your own risk
#	tau -- the desired regression quantile
#	nnzemax -- maximum number of non-zero elements in AA'
#	tmpmax -- upper bound for the dimension of tmpvec
#	cachsz -- size of the cache memory; it's machine dependent
#	small -- the tolerance used to check for convergence
#################################################################
	y <- -y
	n <- length(y)
	m <- a@dimension[2]
	if(n != a@dimension[1]) 
		stop("Dimensions of design matrix and the response vector are not compatible")
	u <- rep(1,length=n)
	x <- rep((1-tau),length=n)
	nnzdmax <- nnza <- a@ia[n+1]-1
	iwmax <- 7*m+3
	ao <- t(a)
	e <- ao%*%a
	nnzemax <- e@ia[m+1]-1
	if(missing(nsubmax)) nsubmax <- nnzemax
	if(missing(nnzlmax)) nnzlmax <- 4*nnzdmax
	if(missing(tmpmax)) tmpmax <- 6*m
	level <- 8
	wwm <- vector("numeric",3*m)
	s <- u - x
	b1 <- solve(e,ao%*%y,tmpmax=tmpmax,nnzlmax=nnzlmax,nsubmax=nsubmax)
	r <- y - a%*%b1
	z <- ifelse(abs(r)<small,(r*(r>0)+small),r*(r>0))
	w <- z - r
	wwn <- matrix(0,n,14)
	wwn[,1] <- r
	wwn[,2] <- z
	wwn[,3] <- w
	srqfnb.o <- .Fortran("srqfn",
		n = as.integer(n),
		m = as.integer(m),
		nnza = as.integer(nnza),
		a = as.double(a@ra),
		ja = as.integer(a@ja),
		ia = as.integer(a@ia),
		ao = as.double(ao@ra),
		jao = as.integer(ao@ja),
		iao = as.integer(ao@ia),
		nnzdmax = as.integer(nnzdmax),
		d = double(nnzdmax),
		jd = integer(nnzdmax),
		id = integer(m+1),
		dsub = double(nnzemax+1),
		jdsub = integer(nnzemax+1),
		nnzemax = as.integer(nnzemax),
		e = as.double(e@ra),
		je = as.integer(e@ja),
		ie = as.integer(e@ia),
		nsubmax = as.integer(nsubmax),
		lindx = integer(nsubmax),	
		xlindx = integer(m+1),
		nnzlmax = as.integer(nnzlmax),
		lnz = double(nnzlmax),
		xlnz = integer(m+1),
		iw = integer(m*5),
		iwmax = as.integer(iwmax),
		iwork = integer(iwmax),
		xsuper = integer(m+1),
		tmpmax = as.integer(tmpmax),
		tmpvec = double(tmpmax),
		wwm = as.double(wwm),
		wwn = as.double(wwn),
		cachsz = as.integer(cachsz),
		level = as.integer(level),
		x = as.double(x),
		s = as.double(s),
		u = as.double(u),
		c = as.double(y),
		sol = as.double(b1),
		rhs = as.double(rhs),
		small = as.double(small),
		ierr = integer(1),
		maxiter = as.integer(maxiter),
		time = double(7),
		PACKAGE = "nprq")
	if(!all(srqfnb.o$ierr==0) && warn.mesg) warning("Check ierr!\n")
	list(coef=-srqfnb.o$sol,ierr=srqfnb.o$ierr,it=srqfnb.o$maxiter,time=sum(srqfnb.o$time))
}
#-------------------------------------------------------------------------------
"rq.fit.sfnc" <- function(x,y,R,r,tau=0.5, rhs = (1-tau)*c(t(x) %*% rep(1,length(y))),
	nsubmax,tmpmax,nnzlmax,cachsz=64,small=1.e-8,maxiter=100,warn.mesg=TRUE,...){
#################################################################
# Interface for a sparse implementation of LMS interior point method
#	x -- structure of A1' stored in csr format
#	y -- pseudo response vector
#	m -- column dimension of A1' in full mode
#       R -- structure of the constraint matrix A2' stored in csr format
#	r -- rhs of the inequality constraints
#	tau -- the desired regression quantile
#       rhs -- the rhs of the dual problem -- specify at your own risk
#	nnzhmax -- upper bound of the non-zero elements in AA'
#       cachsz -- size of the cache memory; it's machine dependent
#       small -- the tolerance used to check for convergence
#################################################################
	y <- -y
	r <- -r
	n1 <- length(y)
	m <- x@dimension[2]
	if(n1 != x@dimension[1]) stop("The design matrix A1' and response vector y are not compatible")
	n2 <- length(r)
	if(n2 != R@dimension[1]) stop("The constraint matrix A2' and the constraint right-hand-side are not compatible")
	maxn1n2 <- max(n1,n2)
	u <- rep(1,length=n1)
	x1 <- rep(1-tau,length=n1)
	x2 <- rep(1,length=n2)
	wwm <- vector("numeric",6*m)
	wwm[1:m] <- rhs
	nnzx <- x@ia[x@dimension[1]+1]-1
	nnzR <- R@ia[R@dimension[1]+1]-1
	nnzdmax <- max(nnzx,nnzR)
	iwmax <- 7*m+3
	ao1 <- t(x)
	ao2 <- t(R)
	e <- ao1%*%x
	g <- ao2%*%R
	h <- e + g
	nnzemax <- e@ia[e@dimension[1]+1]-1
	nnzgmax <- g@ia[g@dimension[1]+1]-1
	nnzhmax <- h@ia[h@dimension[1]+1]-1
	if(missing(nnzlmax)) nnzlmax <- 4*nnzdmax 
	if(missing(nsubmax)) nsubmax <- nnzhmax
	if(missing(tmpmax)) tmpmax <- 6*m
	s <- u - x1
	chol.o <- chol(ao1%*%x,tmpmax=tmpmax,nsubmax=nsubmax,nnzlmax=nnzlmax)
	b <- backsolve(chol.o,ao1%*%y)
	r1 <- y - x%*%b
	z1 <- ifelse(abs(r1)<small,(r1*(r1>0)+small),r1*(r1>0))
	w <- z1 - r1
	z2 <- rep(1,n2)
	wwn1 <- matrix(0,n1,10)
	wwn1[,1] <- z1
	wwn1[,2] <- w
	wwn2 <- matrix(0,n2,7)
	wwn2[,2] <- z2
	level <- 8
	srqfnc.o <- .Fortran("srqfnc",
		n1 = as.integer(n1),
		m = as.integer(m),
		nnzx = as.integer(nnzx),
		x = as.double(x@ra),
		jx = as.integer(x@ja),
		ix = as.integer(x@ia),
		ao1 = as.double(ao1@ra),
		jao1 = as.integer(ao1@ja),
		iao1 = as.integer(ao1@ia),
		n2 = as.integer(n2),
		nnzR = as.integer(nnzR),
		R = as.double(R@ra),
		jR = as.integer(R@ja),
		iR = as.integer(R@ia),
		ao2 = as.double(ao2@ra),
		jao2 = as.integer(ao2@ja),
		iao2 = as.integer(ao2@ia),
		nnzdmax = as.integer(nnzdmax),
		d = double(nnzdmax),
		jd = integer(nnzdmax),
		id = integer(m+1),
		dsub = double(nnzhmax+1),
		jdsub = integer(nnzhmax+1),
		nnzemax = as.integer(nnzemax),
		e = as.double(e@ra),
		je = as.integer(e@ja),
		ie = as.integer(e@ia),
		nnzgmax = as.integer(nnzgmax),
		g = double(nnzgmax),
		jg = integer(nnzgmax),
		ig = integer(m+1),
		nnzhmax = as.integer(nnzhmax),
		h = double(nnzhmax),
		jh = integer(nnzhmax),
		ih = integer(m+1),
		nsubmax = as.integer(nsubmax),
		lindx = integer(nsubmax),	
		xlindx = integer(m+1),
		nnzlmax = as.integer(nnzlmax),
		lnz = double(nnzlmax),
		xlnz = integer(m+1),
		iw = integer(m*5),
		iwmax = as.integer(iwmax),
		iwork = integer(iwmax),
		xsuper = integer(m+1),
		tmpmax = as.integer(tmpmax),
		tmpvec = double(tmpmax),
		maxn1n2 = as.integer(maxn1n2),
		ww1 = double(maxn1n2),
		wwm = as.double(wwm),
		wwn1 = as.double(wwn1),
		wwn2 = as.double(wwn2),
		cachsz = as.integer(cachsz),
		level = as.integer(level),
		x1 = as.double(x1),
		x2 = as.double(x2),
		s = as.double(s),
		u = as.double(u),
		c1 = as.double(y),
		c2 = as.double(r),
		sol = as.double(b),
		small = as.double(small),
		ierr = integer(1),
		maxiter = as.integer(maxiter),
		time = double(7),
		PACKAGE = "nprq")
	if(srqfnc.o$ierr==13 && warn.mesg) stop("Increase nnzh.factor")
	if(!all(srqfnc.o$ierr==0) && warn.mesg) warning("Check ierr!\n")
	list(coef=-srqfnc.o$sol,ierr=srqfnc.o$ierr,it=srqfnc.o$maxiter,time=sum(srqfnc.o$time))
}
