\name{CobarOre}
\alias{CobarOre}
\docType{data}
\title{ Cobar Ore data }
\description{
Cobar Ore data from Green and Silverman (1994). 
The data consists of measurements on the "true width"
of an ore-bearing rock layer from a mine in Cobar, Australia.
}
\usage{data(CobarOre)}
\format{
  A data frame with 38 observations on the following 3 variables.
  \describe{
    \item{x}{x-coordinate of location of mine site}
    \item{y}{y-coordinate of location of mine site}
    \item{z}{ore thickness}
  }
}
\source{
Green, P.J. and B.W. Silverman (1994) Nonparametric Regression Generalized Linear Models:
A roughness penalty approach, Chapman Hall.
}
\examples{
data(CobarOre)
plot(CobarOre)
}
\keyword{datasets}
\eof
\name{lprq}
\alias{lprq}
\title{ locally polynomial quantile regression }
\description{
This is a toy function to illustrate how to do locally polynomial
quantile regression univariate smoothing.  
}
\usage{
lprq(x, y, h, tau = .5, m = 50)
}
\arguments{
  \item{x}{ The conditioning covariate}
  \item{y}{ The response variable  }
  \item{h}{ The bandwidth parameter }
  \item{tau}{ The quantile to be estimated }
  \item{m}{ The number of points at which the function is to be estimated }
}
\details{
The function obviously only does locally linear fitting but can be easily
adapted to locally polynomial fitting of higher order.  The author doesn't
really approve of this sort of smoothing, being more of a spline person,
so the code is left is its (almost) most trivial form. 
}
\value{
The function compute a locally weighted linear quantile regression fit
at each of the m design points, and returns:
  \item{xx}{The design points at which the evaluation occurs}
  \item{fv}{The estimated function values at these design points}
  \item{dev}{The estimated first derivative values at the design points}
}
\references{ Koenker, R. (2004) Quantile Regression }
\author{R. Koenker }
\note{One can also consider using B-spline expansions see \code{bs}.}

\seealso{ \code{rqss} for a general approach to oonparametric QR fitting.  }
\examples{
require(MASS)
data(mcycle)
attach(mcycle)
plot(times,accel,xlab = "milliseconds", ylab = "acceleration (in g)")
hs <- c(1,2,3,4)
for(i in hs){
        h = hs[i]
        fit <- lprq(times,accel,h=h,tau=.5)
        lines(fit$xx,fit$fv,lty=i)
        }
legend(50,-70,c("h=1","h=2","h=3","h=4"),lty=1:length(hs))
}
\keyword{smooth}
\keyword{robust}
\eof
\name{plot}
\alias{plot.rqss}
\alias{plot.qss1}
\alias{plot.qss2}
\title{Default Ploting Method for rqss()}
\description{
Takes a fitted \code{rqss} object produced by \code{rqss()} and plots
the component smooth functions that make up the ANOVA decomposition.
Since the components "omit the intercept" the estimated intercept is added back 
in -- this facilitates the comparison of quantile fits particularly.
}
\usage{
plot.rqss(x, ...)
}
\arguments{
  \item{x}{a fitted \code{rqss} object produced by \code{rqss()}.}
  \item{...}{
    additional arguments for the plotting algorithm}
}
\value{
    The function produces plots for the ANOVA components as a side effect.
}
\author{ Roger Koenker }
\seealso{ \code{\link{rqss}}}
\examples{
n <- 200
x <- sort(rchisq(n,4))
z <- x + rnorm(n)
y <- log(x)+ .1*(log(x))^2 + log(x)*rnorm(n)/4 + z
plot(x,y-z)
fit <- rqss(y~qss(x,constraint="N")+z)
lines(x[-1],fit$coef[1]+fit$coef[-(1:2)])
fit <- rqss(y~qss(x,constraint="I")+z)
lines(x[-1],fit$coef[1]+fit$coef[-(1:2)],col="blue")
fit <- rqss(y~qss(x,constraint="CI")+z)
lines(x[-1],fit$coef[1]+fit$coef[-(1:2)],col="red")
#Cleanup
rm(list=ls())
#A bivariate example
data(CobarOre)
attach(CobarOre)
fit <- rqss(z~qss(cbind(x,y),lambda=.08))
plot(fit)
}
\keyword{regression}
\keyword{smooth}
\keyword{iplot}
\eof
\name{qss}
\alias{qss}
\alias{qss1}
\alias{qss2}
\alias{triogram.fidelity}
\alias{triogram.penalty}
\title{Additive Nonparametric Terms for rqss Fitting}
\description{
In the formula specification of \code{rqss} nonparametric terms 
are specified with \code{qss}.  Both univariate and bivariate
specifications are possible, and qualitative constraints may also be specified
for the qss terms.
}
\usage{
qss(x, constraint = "N", lambda = 1, w = rep(1, length(x)))
}
\arguments{
  \item{x}{The covariate determining the nonparametric component, if
	x is a matrix with two columns then  the qss
	function will construct a penalized triogram term.}
  \item{lambda}{The smoothing parameter governing the tradeoff between
	fidelity and the penalty component for this term.  In future
	versions there should be an automatic mechanism for default
	choice of the lambdas.  For now, this is the responsibility
	of the user.}
  \item{constraint}{Optional specification of qualitative constraints 
	on the fitted univariate qss functions, take the values: "N","I","D","U","C"
	"UI","UD","CI","CD" for none, increasing, decreasing, convex,
	concave, convex and increasing, etc.  And for bivariate qss
	components can take the values "N","U","C" for none, convex, and concave.} 
  \item{w}{ weights not yet unimplemented }
}
\details{
The various pieces returned are stored in sparse matrix.csr form.
See \code{rqss} for details on how they are assembled.  To preserve the
sparsity of the design matrix the first column of each qss term is dropped.
This differs from the usual convention that would have forced qss terms
to have mean zero.  This convention has implications for prediction that need to be
recognized.  The penalty components for qss terms are based on total
variation penalization of the first derivative (and gradient, for bivariate x)
as described in the references appearing in the help for \code{rqss}.
}
\value{
  \item{F}{Fidelity component of the design matrix}
  \item{A}{Penalty component of the design matrix}
  \item{R}{Constraint component of the design matrix}
  \item{r}{Constraint component of the rhs}
}
\author{Roger Koenker}

\seealso{ \code{\link{rqss}}}
\keyword{smooth}
\keyword{robust}
\eof
\name{rq.fit.sfn}
\alias{rq.fit.sfn}
\title{Sparse Regression Quantile Fitting}
\description{
Fit a quantile regression model using a sparse implementation of the Frisch-Newton
interior-point algorithm.
}
\usage{
rq.fit.sfn(a, y, tau = 0.5, rhs = (1-tau)*c(t(a) %*% rep(1,length(y))), 
   nsubmax, tmpmax, nnzlmax, cachsz = 64, small = 1e-06, maxiter = 100, warn.mesg = T, ...)
}
\arguments{
  \item{a}{structure of the design matrix X stored in csr format}
  \item{y}{response vector}
  \item{tau}{desired quantile}
  \item{rhs}{the right-hand-side of the dual problem; regular users shouldn't need to specify this}
  \item{nsubmax}{upper bound of the dimension of lindx in Cholesky factorization;computed automatically inside the routine if missing}
  \item{tmpmax}{upper bound of the working array in Cholesky factorization;computed automatically inside the routine if missing}
  \item{nnzlmax}{upper bound of the non-zero entries in the Cholesky factor L;computed automatically inside the routine if missing}
  \item{cachsz}{size of the cache on the machine; default to 64}
  \item{small}{convergence tolerance for the interior algorithm}
  \item{maxiter}{upper limit for the number of iterations}
  \item{warn.mesg}{flag for printing of warning message; default to TRUE}
  \item{\dots}{optional arguments}
}
\details{
This is a sparse implementation of the Frisch-Newton algorithm for quantile
regression described in Koenker and Portnoy (1996). The sparse matrix 
linear algebra is implemented through the functions available in the R packages 
\code{SparseM}.
}
\value{
  \item{coef }{Regression quantile coefficients}
  \item{ierr }{Error code for the internal Fortran routine \code{srqfnc}:
                \describe{
                        \item{1:}{ insufficient work space in call to \code{extrac
t}}
                        \item{2:}{ nnzd > nnzdmax}
			\item{3:}{ insufficient storage in iwork when calling ordmmd}
			\item{4:}{ insufficient storage in iwork when calling sfinit}
			\item{5:}{ nnzl > nnzlmax when calling sfinit}
			\item{6:}{ nsub > nsubmax when calling sfinit}
			\item{7:}{ insufficient work space in iwork when calling symfct}
			\item{8:}{ inconsistancy in input when calling symfct}
			\item{9:}{ tmpsiz > tmpmax when calling bfinit; increase tmpmax}
			\item{10:}{ nonpositive diagonal encountered, not positive definite}
			\item{11:}{ insufficient work storage in tmpvec when calling blkfct}
			\item{12:}{ insufficient work storage in iwork when calling blkfct}
			}
		}
  \item{it}{Iteration count}
  \item{time}{Amount of time used in the computation}
}
\references{
Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R, \cr
\url{http://www.econ.uiuc.edu/~roger/research}
}
\author{Pin Ng}
\seealso{\code{rq.fit.sfnc} for the constrained version,
\code{SparseM} for a sparse matrix package for \R
}

\examples{
n=200
p=50
X=rnorm(n*p)
X[abs(X)<2.0]=0
X=cbind(1,matrix(X,n,p))
y=.5*apply(X,1,sum)+rnorm(n)
rq.o=rq.fit.sfn(as.matrix.csr(X),y,tmpmax=floor(1e5+exp(-12.1)*(as.matrix.csr(X)@ia[p+1]-1)^2.35))
}
\keyword{regression}
\eof
\name{rq.fit.sfnc}
\alias{rq.fit.sfnc}
\title{Sparse Constrained Regression Quantile Fitting}
\description{
Fit constrained regression quantiles using a sparse implementation of the Frisch-Newton
Interior-point algorithm.
}
\usage{
rq.fit.sfnc(x, y, R, r, tau, rhs = (1-tau)*c(t(a) %*% rep(1,length(y))), 
   nsubmax, tmpmax, nnzlmax, cachsz = 64, small = 1e-08, maxiter = 100, warn.mesg = T, ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{structure of the design matrix X stored in csr format}
  \item{y}{response vector}
  \item{R}{constraint matrix stored in csr format}
  \item{r}{right-hand-side of the constraint}
  \item{tau}{desired quantile}
  \item{rhs}{the right-hand-side of the dual problem; regular users shouldn't need to specify this}
  \item{nsubmax}{upper bound of the dimension of lindx in Cholesky factorization;computed automatically inside the routine if missing}
  \item{tmpmax}{upper bound of the working array in Cholesky factorization;computed automatically inside the routine if missing}
  \item{nnzlmax}{upper bound of the non-zero entries in the Cholesky factor L;computed automatically inside the routine if missing}
  \item{cachsz}{size of the cache on the machine; default to 64}
  \item{small}{convergence tolerance for the interior algorithm}
  \item{maxiter}{upper limit for the number of iterations}
  \item{warn.mesg}{flag for printing of warning message; default to TRUE}
  \item{\dots}{additional arguments}
}
\details{
This is a sparse implementation of the Frisch-Newton algorithm for 
constrained quantile regression described in Koenker and Portnoy (1996). 
The sparse matrix
linear algebra is implemented through the functions available in the R packages
\code{SparseM}.
}
\value{
  \item{coef }{Regression quantile coefficients}
  \item{ierr }{Error code for the internal Fortran routine \code{srqfn}: 
	\describe{
		\item{1:}{ insufficient work space in call to \code{extract}}
	        \item{3:}{ insufficient storage in iwork when calling ordmmd}
      		\item{4:}{ insufficient storage in iwork when calling sfinit}
		\item{5:}{ nnzl > nnzlmax when calling sfinit}
		\item{6:}{ nsub > nsubmax when calling sfinit}
		\item{7:}{ insufficient work space in iwork when calling symfct}
		\item{8:}{ inconsistancy in input when calling symfct}
		\item{9:}{ tmpsiz > tmpmax when calling symfct; increase tmpmax}
		\item{10:}{ nonpositive diagonal encountered when calling blkfct}
		\item{11:}{ insufficient work storage in tmpvec when calling blkfct}
		\item{12:}{ insufficient work storage in iwork when calling blkfct}
		\item{13:}{ nnzd > nnzdmax in e,je when calling amub}
		\item{14:}{ nnzd > nnzdmax in g,jg when calling amub}
		\item{15:}{ nnzd > nnzdmax in h,jh when calling aplb}
			}
		}
  \item{it}{Iteration count}
  \item{time}{Amount of time used in the computation}

}
\references{Koenker, R and Ng, P. (2002).  SparseM:  A Sparse Matrix Package for \R, \cr
\url{http://www.econ.uiuc.edu/~roger/research}
}
\author{Pin Ng}
\seealso{\code{rq.fit.sfn} for the unconstrained version,
\code{SparseM} for a sparse matrix package for \R
}
\examples{
n=200
p=50
X=rnorm(n*p)
X[abs(X)<2.0]=0
X=cbind(1,matrix(X,n,p))
y=.5*apply(X,1,sum)+rnorm(n)
R=rbind(diag(p+1),-diag(p+1))
r=c(rep(0,p+1),-rep(1,p+1))
rq.o=rq.fit.sfnc(as.matrix.csr(X),y,as.matrix.csr(R),r,tmpmax=floor(1e5+exp(-12.1)*(as.matrix.csr(X)@ia[p+1]-1)^2.35))
}

\keyword{regression}
\eof
\name{rqss}
\alias{rqss}
\alias{rqss.fit}
\alias{[.terms}
\alias{untangle.specials}
\title{Additive Quantile Regression Smoothing}
\description{
Fitting function for additive quantile regression models with possible univariate
and/or bivariate nonparametric terms estimated by total variation regularization.
}
\usage{
rqss(formula, tau = 0.5, data = parent.frame(), weights, na.action, 
	method = "fn", contrasts = NULL, ...)
}
\arguments{
  \item{formula}{  a formula object, with the response on the left of a `~'
        operator,  and terms, separated by `+' operators, on the right.
	The terms may include \code{qss} terms that represent additive
	nonparametric components.  These terms can be univariate or
	bivariate.  See \code{qss} for details on how to
	specify these terms.}
  \item{tau}{
    the quantile to be estimated, this must be a number between 0 and 1,
  }
  \item{data}{
    a data.frame in which to interpret the variables
    named in the formula, or in the subset and the weights argument.
  }
 \item{weights}{
    vector of observation weights; if supplied, the algorithm fits
    to minimize the sum of the weights multiplied into the
    absolute residuals. The length of weights must be the same as
    the number of observations.  The weights must be nonnegative
    and it is strongly recommended that they be strictly positive,
    since zero weights are ambiguous.
  }
  \item{na.action}{
    a function to filter missing data.
    This is applied to the model.frame after any subset argument has been used.
    The default (with \code{na.fail}) is to create an error if any missing values are
    found.  A possible alternative is \code{na.omit}, which
    deletes observations that contain one or more missing values.
  }
  \item{method}{
    the algorithmic method used to compute the fit.  There are currently
    two options.   Both are implementations of the Frisch--Newton interior 
    point method described in detail in Portnoy and Koenker(1997).   Both
    are implemented using sparse Cholesky decomposition as described in
    Koenker and Ng (2003)
    Option \code{"sfnc"} is used if the user specifies inequality constraints.
    Option \code{"sfn"} is used if there are no inequality constraints.
    Linear inequality constraints on the fitted coefficients are specified
    by a matrix \code{R} and a vector \code{r}, specified inside the \code{qss}
    terms, representing the constraints in the form $Rb \geq r$.} 

  \item{contrasts}{
    a list giving contrasts for some or all of the factors
    default = \code{NULL} appearing in the model formula.
    The elements of the list should have the same name as the variable
    and should be either a contrast matrix (specifically, any full-rank
    matrix with as many rows as there are levels in the factor),
    or else a function to compute such a matrix given the number of levels.
  }
  \item{...}{
    additional arguments for the fitting routines
    (see \code{\link{rq.fit.sfn}} and \code{\link{rq.fit.sfnc}} 
    and the functions they call).  }
}
\details{ Total variation regularization for univariate and
    bivariate nonparametric quantile smoothing is described
    in Koenker, Ng and Portnoy (1994) and Koenker and Mizera(2003)
    respectively.  The additive model extension of this approach
    depends crucially on the sparse linear algebra implementation
    for R described in Koenker and Ng (2003).  }
\value{
    The function returns a fitted object representing the estimated
    model specified in the formula.  See \code{rqss.object} 
    for further details on this object, and references to methods
    to look at it.
}
\references{
[1] Koenker, R. and S. Portnoy (1997) The Gaussian Hare and the Laplacean
Tortoise:  Computability of Squared-error vs Absolute Error Estimators,
(with discussion).  \emph{Statistical Science,} \bold{12}, 279-300.

[2] Koenker, R., P. Ng and S. Portnoy, (1994)  Quantile Smoothing Splines'' 
\emph{Biometrika}, 81, 673-680.

[3] Koenker, R. and I. Mizera, (2003)   Penalized Triograms: Total Variation
Regularization for Bivariate Smoothing, \emph{JRSS(B)}, forthcoming.

[4] Koenker, R. and  P. Ng (2003)  SparseM:  A Sparse Linear Algebra Package for R,
\emph{J. Stat. Software}.
}
\author{ Roger Koenker }
\seealso{ \code{\link{qss}}}
\examples{
n <- 200
x <- sort(rchisq(n,4))
z <- x + rnorm(n)
y <- log(x)+ .1*(log(x))^2 + log(x)*rnorm(n)/4 + z
plot(x,y-z)
fit <- rqss(y~qss(x,constraint="N")+z)
lines(x[-1],fit$coef[1]+fit$coef[-(1:2)])
fit <- rqss(y~qss(x,constraint="I")+z)
lines(x[-1],fit$coef[1]+fit$coef[-(1:2)],col="blue")
fit <- rqss(y~qss(x,constraint="CI")+z)
lines(x[-1],fit$coef[1]+fit$coef[-(1:2)],col="red")
#Cleanup
rm(list=ls())
#A bivariate example
data(CobarOre)
attach(CobarOre)
fit <- rqss(z~qss(cbind(x,y),lambda=.08))
plot(fit)
}
\keyword{regression}
\keyword{smooth}
\keyword{robust}
\eof
